@@ -1,8 +1,5 @@
-from __future__ import absolute_import, print_function
-
 import os
 
-from cryptography.hazmat.backends import default_backend
 from cryptography.hazmat.primitives import hashes
 from cryptography.hazmat.primitives.asymmetric import ec
 from cryptography.hazmat.primitives.asymmetric.utils import (
@@ -29,18 +26,18 @@ def verify_one_vector(vector):
 
     numbers = ec.EllipticCurvePublicNumbers(x, y, ec.SECP256K1())
 
-    key = numbers.public_key(default_backend())
+    key = numbers.public_key()
 
     verifier = key.verifier(
         signature, ec.ECDSA(CRYPTOGRAPHY_HASH_TYPES[digest_algorithm]())
     )
     verifier.update(message)
-    return verifier.verify()
+    verifier.verify()
 
 
 def verify_vectors(vectors):
     for vector in vectors:
-        assert verify_one_vector(vector)
+        verify_one_vector(vector)
 
 
 vector_path = os.path.join("asymmetric", "ECDSA", "SECP256K1", "SigGen.txt")
