@@ -10,13 +10,13 @@ INCLUDES = """
 TYPES = """
 typedef ... BIO;
 typedef ... BIO_METHOD;
+typedef ... BIO_ADDR;
 """
 
 FUNCTIONS = """
 int BIO_free(BIO *);
 void BIO_free_all(BIO *);
 BIO *BIO_new_file(const char *, const char *);
-BIO *BIO_new_dgram(int, int);
 size_t BIO_ctrl_pending(BIO *);
 int BIO_read(BIO *, void *, int);
 int BIO_gets(BIO *, char *, int);
@@ -25,8 +25,7 @@ int BIO_write(BIO *, const void *, int);
 int BIO_up_ref(BIO *);
 
 BIO *BIO_new(BIO_METHOD *);
-BIO_METHOD *BIO_s_mem(void);
-BIO_METHOD *BIO_s_datagram(void);
+const BIO_METHOD *BIO_s_mem(void);
 BIO *BIO_new_mem_buf(const void *, int);
 long BIO_set_mem_eof_return(BIO *, int);
 long BIO_get_mem_data(BIO *, char **);
@@ -37,7 +36,23 @@ int BIO_should_retry(BIO *);
 int BIO_reset(BIO *);
 void BIO_set_retry_read(BIO *);
 void BIO_clear_retry_flags(BIO *);
+
+BIO_ADDR *BIO_ADDR_new(void);
+void BIO_ADDR_free(BIO_ADDR *);
 """
 
 CUSTOMIZATIONS = """
+#if CRYPTOGRAPHY_IS_LIBRESSL || CRYPTOGRAPHY_IS_BORINGSSL
+#include <sys/socket.h>
+#include <stdlib.h>
+typedef struct sockaddr BIO_ADDR;
+
+BIO_ADDR *BIO_ADDR_new(void) {
+    return malloc(sizeof(struct sockaddr_storage));
+}
+
+void BIO_ADDR_free(BIO_ADDR *ptr) {
+    free(ptr);
+}
+#endif
 """
