@@ -2,18 +2,12 @@
 # 2.0, and the BSD License. See the LICENSE file in the root of this repository
 # for complete details.
 
-from __future__ import absolute_import, division, print_function
 
 INCLUDES = """
 #include <openssl/ecdsa.h>
 """
 
 TYPES = """
-typedef ... ECDSA_SIG;
-
-typedef ... CRYPTO_EX_new;
-typedef ... CRYPTO_EX_dup;
-typedef ... CRYPTO_EX_free;
 """
 
 FUNCTIONS = """
