@@ -18,11 +18,16 @@ from cryptography.exceptions import (
 )
 from cryptography.hazmat.primitives import hashes, hmac, serialization
 from cryptography.hazmat.primitives.asymmetric import rsa
-from cryptography.hazmat.primitives.ciphers import Cipher
+from cryptography.hazmat.primitives.ciphers import (
+    BlockCipherAlgorithm,
+    Cipher,
+    algorithms,
+)
 from cryptography.hazmat.primitives.ciphers.modes import GCM
 from cryptography.hazmat.primitives.kdf.hkdf import HKDF, HKDFExpand
 from cryptography.hazmat.primitives.kdf.kbkdf import (
     CounterLocation,
+    KBKDFCMAC,
     KBKDFHMAC,
     Mode,
 )
@@ -43,10 +48,8 @@ def _load_all_params(path, file_names, param_loader):
 def generate_encrypt_test(
     param_loader, path, file_names, cipher_factory, mode_factory
 ):
-    all_params = _load_all_params(path, file_names, param_loader)
-
     def test_encryption(self, backend, subtests):
-        for params in all_params:
+        for params in _load_all_params(path, file_names, param_loader):
             with subtests.test():
                 encrypt_test(backend, cipher_factory, mode_factory, params)
 
@@ -76,13 +79,12 @@ def encrypt_test(backend, cipher_factory, mode_factory, params):
 def generate_aead_test(
     param_loader, path, file_names, cipher_factory, mode_factory
 ):
-    all_params = _load_all_params(path, file_names, param_loader)
-
     assert mode_factory is GCM
-    # We don't support IVs < 64-bit in GCM mode so just strip them out
-    all_params = [i for i in all_params if len(i["iv"]) >= 16]
 
     def test_aead(self, backend, subtests):
+        all_params = _load_all_params(path, file_names, param_loader)
+        # We don't support IVs < 64-bit in GCM mode so just strip them out
+        all_params = [i for i in all_params if len(i["iv"]) >= 16]
         for params in all_params:
             with subtests.test():
                 aead_test(backend, cipher_factory, mode_factory, params)
@@ -101,23 +103,25 @@ def aead_test(backend, cipher_factory, mode_factory, params):
         # hex encoded.
         pytest.skip("Non-96-bit IVs unsupported in FIPS mode.")
 
+    mode = mode_factory(
+        binascii.unhexlify(params["iv"]),
+        binascii.unhexlify(params["tag"]),
+        len(binascii.unhexlify(params["tag"])),
+    )
+    assert isinstance(mode, GCM)
     if params.get("pt") is not None:
-        plaintext = params["pt"]
-    ciphertext = params["ct"]
-    aad = params["aad"]
+        plaintext = binascii.unhexlify(params["pt"])
+    ciphertext = binascii.unhexlify(params["ct"])
+    aad = binascii.unhexlify(params["aad"])
     if params.get("fail") is True:
         cipher = Cipher(
             cipher_factory(binascii.unhexlify(params["key"])),
-            mode_factory(
-                binascii.unhexlify(params["iv"]),
-                binascii.unhexlify(params["tag"]),
-                len(binascii.unhexlify(params["tag"])),
-            ),
+            mode,
             backend,
         )
         decryptor = cipher.decryptor()
-        decryptor.authenticate_additional_data(binascii.unhexlify(aad))
-        actual_plaintext = decryptor.update(binascii.unhexlify(ciphertext))
+        decryptor.authenticate_additional_data(aad)
+        actual_plaintext = decryptor.update(ciphertext)
         with pytest.raises(InvalidTag):
             decryptor.finalize()
     else:
@@ -127,8 +131,8 @@ def aead_test(backend, cipher_factory, mode_factory, params):
             backend,
         )
         encryptor = cipher.encryptor()
-        encryptor.authenticate_additional_data(binascii.unhexlify(aad))
-        actual_ciphertext = encryptor.update(binascii.unhexlify(plaintext))
+        encryptor.authenticate_additional_data(aad)
+        actual_ciphertext = encryptor.update(plaintext)
         actual_ciphertext += encryptor.finalize()
         tag_len = len(binascii.unhexlify(params["tag"]))
         assert binascii.hexlify(encryptor.tag[:tag_len]) == params["tag"]
@@ -142,19 +146,17 @@ def aead_test(backend, cipher_factory, mode_factory, params):
             backend,
         )
         decryptor = cipher.decryptor()
-        decryptor.authenticate_additional_data(binascii.unhexlify(aad))
-        actual_plaintext = decryptor.update(binascii.unhexlify(ciphertext))
+        decryptor.authenticate_additional_data(aad)
+        actual_plaintext = decryptor.update(ciphertext)
         actual_plaintext += decryptor.finalize()
-        assert actual_plaintext == binascii.unhexlify(plaintext)
+        assert actual_plaintext == plaintext
 
 
 def generate_stream_encryption_test(
     param_loader, path, file_names, cipher_factory
 ):
-    all_params = _load_all_params(path, file_names, param_loader)
-
     def test_stream_encryption(self, backend, subtests):
-        for params in all_params:
+        for params in _load_all_params(path, file_names, param_loader):
             with subtests.test():
                 stream_encryption_test(backend, cipher_factory, params)
 
@@ -180,10 +182,8 @@ def stream_encryption_test(backend, cipher_factory, params):
 
 
 def generate_hash_test(param_loader, path, file_names, hash_cls):
-    all_params = _load_all_params(path, file_names, param_loader)
-
     def test_hash(self, backend, subtests):
-        for params in all_params:
+        for params in _load_all_params(path, file_names, param_loader):
             with subtests.test():
                 hash_test(backend, hash_cls, params)
 
@@ -235,10 +235,8 @@ def base_hmac_test(backend, algorithm):
 
 
 def generate_hmac_test(param_loader, path, file_names, algorithm):
-    all_params = _load_all_params(path, file_names, param_loader)
-
     def test_hmac(self, backend, subtests):
-        for params in all_params:
+        for params in _load_all_params(path, file_names, param_loader):
             with subtests.test():
                 hmac_test(backend, algorithm, params)
 
@@ -253,10 +251,8 @@ def hmac_test(backend, algorithm, params):
 
 
 def generate_pbkdf2_test(param_loader, path, file_names, algorithm):
-    all_params = _load_all_params(path, file_names, param_loader)
-
     def test_pbkdf2(self, backend, subtests):
-        for params in all_params:
+        for params in _load_all_params(path, file_names, param_loader):
             with subtests.test():
                 pbkdf2_test(backend, algorithm, params)
 
@@ -286,9 +282,11 @@ def generate_aead_exception_test(cipher_factory, mode_factory):
 
 
 def aead_exception_test(backend, cipher_factory, mode_factory):
+    mode = mode_factory(binascii.unhexlify(b"0" * 24))
+    assert isinstance(mode, GCM)
     cipher = Cipher(
         cipher_factory(binascii.unhexlify(b"0" * 32)),
-        mode_factory(binascii.unhexlify(b"0" * 24)),
+        mode,
         backend,
     )
     encryptor = cipher.encryptor()
@@ -304,15 +302,18 @@ def aead_exception_test(backend, cipher_factory, mode_factory):
         encryptor.update(b"b" * 16)
     with pytest.raises(AlreadyFinalized):
         encryptor.finalize()
+
+    mode2 = mode_factory(binascii.unhexlify(b"0" * 24), b"0" * 16)
+    assert isinstance(mode2, GCM)
     cipher = Cipher(
         cipher_factory(binascii.unhexlify(b"0" * 32)),
-        mode_factory(binascii.unhexlify(b"0" * 24), b"0" * 16),
+        mode2,
         backend,
     )
     decryptor = cipher.decryptor()
     decryptor.update(b"a" * 16)
     with pytest.raises(AttributeError):
-        decryptor.tag
+        decryptor.tag  # type: ignore[attr-defined]
 
 
 def generate_aead_tag_exception_test(cipher_factory, mode_factory):
@@ -332,6 +333,13 @@ def aead_tag_exception_test(backend, cipher_factory, mode_factory):
     with pytest.raises(ValueError):
         mode_factory(binascii.unhexlify(b"0" * 24), b"000")
 
+    with pytest.raises(ValueError):
+        Cipher(
+            cipher_factory(binascii.unhexlify(b"0" * 32)),
+            mode_factory(binascii.unhexlify(b"0" * 24), b"toolong" * 12),
+            backend,
+        )
+
     with pytest.raises(ValueError):
         mode_factory(binascii.unhexlify(b"0" * 24), b"000000", 2)
 
@@ -386,10 +394,8 @@ def hkdf_expand_test(backend, algorithm, params):
 
 
 def generate_hkdf_test(param_loader, path, file_names, algorithm):
-    all_params = _load_all_params(path, file_names, param_loader)
-
     def test_hkdf(self, backend, subtests):
-        for params in all_params:
+        for params in _load_all_params(path, file_names, param_loader):
             with subtests.test():
                 hkdf_extract_test(backend, algorithm, params)
             with subtests.test():
@@ -401,18 +407,16 @@ def generate_hkdf_test(param_loader, path, file_names, algorithm):
 
 
 def generate_kbkdf_counter_mode_test(param_loader, path, file_names):
-    all_params = _load_all_params(path, file_names, param_loader)
-
     def test_kbkdf(self, backend, subtests):
-        for params in all_params:
+        for params in _load_all_params(path, file_names, param_loader):
             with subtests.test():
                 kbkdf_counter_mode_test(backend, params)
 
     return test_kbkdf
 
 
-def kbkdf_counter_mode_test(backend, params):
-    supported_algorithms: typing.Dict[
+def _kbkdf_hmac_counter_mode_test(backend, prf, ctr_loc, brk_loc, params):
+    supported_hash_algorithms: typing.Dict[
         str, typing.Type[hashes.HashAlgorithm]
     ] = {
         "hmac_sha1": hashes.SHA1,
@@ -422,24 +426,9 @@ def kbkdf_counter_mode_test(backend, params):
         "hmac_sha512": hashes.SHA512,
     }
 
-    supported_counter_locations = {
-        "before_fixed": CounterLocation.BeforeFixed,
-        "after_fixed": CounterLocation.AfterFixed,
-    }
-
-    algorithm = supported_algorithms.get(params.get("prf"))
-    if algorithm is None or not backend.hmac_supported(algorithm()):
-        pytest.skip(
-            "KBKDF does not support algorithm: {}".format(params.get("prf"))
-        )
-
-    ctr_loc = supported_counter_locations.get(params.get("ctrlocation"))
-    if ctr_loc is None or not isinstance(ctr_loc, CounterLocation):
-        pytest.skip(
-            "Does not support counter location: {}".format(
-                params.get("ctrlocation")
-            )
-        )
+    algorithm = supported_hash_algorithms.get(prf)
+    assert algorithm is not None
+    assert backend.hmac_supported(algorithm())
 
     ctrkdf = KBKDFHMAC(
         algorithm(),
@@ -452,21 +441,87 @@ def kbkdf_counter_mode_test(backend, params):
         None,
         binascii.unhexlify(params["fixedinputdata"]),
         backend=backend,
+        break_location=brk_loc,
     )
 
     ko = ctrkdf.derive(binascii.unhexlify(params["ki"]))
     assert binascii.hexlify(ko) == params["ko"]
 
 
+def _kbkdf_cmac_counter_mode_test(backend, prf, ctr_loc, brk_loc, params):
+    supported_cipher_algorithms: typing.Dict[
+        str, typing.Type[BlockCipherAlgorithm]
+    ] = {
+        "cmac_aes128": algorithms.AES,
+        "cmac_aes192": algorithms.AES,
+        "cmac_aes256": algorithms.AES,
+        "cmac_tdes2": algorithms.TripleDES,
+        "cmac_tdes3": algorithms.TripleDES,
+    }
+
+    algorithm = supported_cipher_algorithms.get(prf)
+    assert algorithm is not None
+
+    # TripleDES is disallowed in FIPS mode.
+    if backend._fips_enabled and algorithm is algorithms.TripleDES:
+        pytest.skip("TripleDES is not supported in FIPS mode.")
+
+    ctrkdf = KBKDFCMAC(
+        algorithm,
+        Mode.CounterMode,
+        params["l"] // 8,
+        params["rlen"] // 8,
+        None,
+        ctr_loc,
+        None,
+        None,
+        binascii.unhexlify(params["fixedinputdata"]),
+        backend=backend,
+        break_location=brk_loc,
+    )
+
+    ko = ctrkdf.derive(binascii.unhexlify(params["ki"]))
+    assert binascii.hexlify(ko) == params["ko"]
+
+
+def kbkdf_counter_mode_test(backend, params):
+    supported_counter_locations = {
+        "before_fixed": CounterLocation.BeforeFixed,
+        "after_fixed": CounterLocation.AfterFixed,
+        "middle_fixed": CounterLocation.MiddleFixed,
+    }
+
+    ctr_loc = supported_counter_locations[params.pop("ctrlocation")]
+    brk_loc = None
+
+    if ctr_loc == CounterLocation.MiddleFixed:
+        assert "fixedinputdata" not in params
+        params["fixedinputdata"] = params.pop(
+            "databeforectrdata"
+        ) + params.pop("dataafterctrdata")
+
+        brk_loc = params.pop("databeforectrlen")
+        assert isinstance(brk_loc, int)
+
+    prf = params.get("prf")
+    assert prf is not None
+    assert isinstance(prf, str)
+    del params["prf"]
+    if prf.startswith("hmac"):
+        _kbkdf_hmac_counter_mode_test(backend, prf, ctr_loc, brk_loc, params)
+    else:
+        assert prf.startswith("cmac")
+        _kbkdf_cmac_counter_mode_test(backend, prf, ctr_loc, brk_loc, params)
+
+
 def generate_rsa_verification_test(
     param_loader, path, file_names, hash_alg, pad_factory
 ):
-    all_params = _load_all_params(path, file_names, param_loader)
-    all_params = [
-        i for i in all_params if i["algorithm"] == hash_alg.name.upper()
-    ]
-
     def test_rsa_verification(self, backend, subtests):
+        all_params = _load_all_params(path, file_names, param_loader)
+        all_params = [
+            i for i in all_params if i["algorithm"] == hash_alg.name.upper()
+        ]
         for params in all_params:
             with subtests.test():
                 rsa_verification_test(backend, params, hash_alg, pad_factory)
@@ -517,3 +572,8 @@ def skip_fips_traditional_openssl(backend, fmt):
         pytest.skip(
             "Traditional OpenSSL key format is not supported in FIPS mode."
         )
+
+
+def skip_signature_hash(backend, hash_alg: hashes.HashAlgorithm):
+    if not backend.signature_hash_supported(hash_alg):
+        pytest.skip(f"{hash_alg} is not a supported signature hash algorithm.")
