@@ -8,14 +8,12 @@ import binascii
 import pytest
 
 from cryptography.exceptions import InvalidSignature
-from cryptography.hazmat.backends.interfaces import CMACBackend
 from cryptography.hazmat.primitives.ciphers.algorithms import AES
 from cryptography.hazmat.primitives.cmac import CMAC
 
 from .utils import wycheproof_tests
 
 
-@pytest.mark.requires_backend_interface(interface=CMACBackend)
 @wycheproof_tests("aes_cmac_test.json")
 def test_aes_cmac(backend, wycheproof):
     key = binascii.unhexlify(wycheproof.testcase["key"])
